package Logic;

import Entity.*;
import Mapper.RoadMapper;
import Utils.GraphUtil;
import Utils.MyBatisUtil;
import org.apache.ibatis.session.SqlSession;

import java.util.ArrayList;
import java.util.LinkedList;

public class CarPath {
    //车牌号
    private String carID;
    //该车经过的点
    private LinkedList<Node> carPath;
    //根据路网关系建立的图
    private Graph graph;

    public CarPath() {
    }

    public Graph getGraph() {
        return graph;
    }

    public void setGraph(Graph graph) {
        this.graph = graph;
    }

    public CarPath(String carID) {
        this.carID = carID;
        carPath = new LinkedList<>();
    }

    public String getCarID() {
        return carID;
    }

    public void setCarID(String carID) {
        this.carID = carID;
    }

    public LinkedList<Node> getCarPath() {
        return carPath;
    }

    public void setCarPath(LinkedList<Node> carPath) {
        this.carPath = carPath;
    }

    /**
     * 得到一辆车的路径
     * @return  可以求得路径，值为true，否则为false
     */
    public boolean getPath(){
        LinkedList<Node> path = new LinkedList<>();
        SqlSession sqlSession = MyBatisUtil.getSqlSessionFactory().openSession();
        RoadMapper roadMapper = sqlSession.getMapper(RoadMapper.class);
        ArrayList<VertexNode> vertexNodes = roadMapper.getVertexNodes();    //顶点坐标

        Graph();    //根据路网数据建立的图
        LinkedList<Node> nodes = roadMapper.getNodes();   //所有交点的坐标
        LinkedList<Device> devices = roadMapper.getDevice(carID);    //一辆车被拍到的设备ID
        //纠正设备坐标
        CorrectDevices correctDevices = new CorrectDevices(nodes,devices);
        devices = correctDevices.AllCorrectDevice();

        //删除差错点
        Check check = new Check(graph,devices);
        if(check.CheckAllDevices()){
            devices = check.getDevices();
        }
        else {
            return false;
        }
        //跟据设备ID求该车的路径
        for(int i = 0;i<devices.size()-1;i++){
            Device device1 = devices.get(i);
            Device device2 = devices.get(i+1);
            Dijkstra dijkstra = new Dijkstra(graph);
            if(dijkstra.getShortPath(device1.getNodeID(), device2.getNodeID())){
                LinkedList<Node> temp = dijkstra.getRoadPath();
                carPath.addAll(temp);
            }

        }
        //删除重复的点
        for(int i = 0;i<carPath.size()-1;i++){
            if(carPath.get(i).getNodeID() == carPath.get(i+1).getNodeID()){
                carPath.remove(i+1);
                i--;
            }
        }
        //通过最短路径进一步对路径进行处理
        FinalPath finalPath = new FinalPath();
        carPath = finalPath.getFinalPath(carPath,graph);
        return true;
    }
    /**
     * 通过在数据库中查询到路网中的点的关系，得到图
     * @return 建立的图
     */
    public Graph Graph(){
        SqlSession sqlSession = MyBatisUtil.getSqlSessionFactory().openSession();
        RoadMapper roadMapper = sqlSession.getMapper(RoadMapper.class);
        ArrayList<VertexNode> vertexNodes = roadMapper.getVertexNodes();
        for(VertexNode vertexNode:vertexNodes){
            vertexNode.setArcNodes(new LinkedList<>());
        }
        LinkedList<RoadNode> roadNodes = roadMapper.getRoadNodes();
        graph = GraphUtil.RoadGraph(vertexNodes,roadNodes);
        return this.graph;
    }
}
