package Logic;

import Entity.ArcNode;
import Entity.Graph;
import Entity.Node;
import Entity.VertexNode;
import Utils.GraphUtil;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

public class Dijkstra {
    //路网关系建立的图
    private Graph graph;
    //一条路的长度
    private  Double length;
    //这条路经过的路网的点
    private LinkedList<Node> roadPath;

    public Dijkstra(Graph graph) {
        this.graph = graph;
        roadPath = new LinkedList<>();
    }

    public Graph getGraph() {
        return graph;
    }

    public void setGraph(Graph graph) {
        this.graph = graph;
    }

    public Double getLength() {
        return length;
    }

    public void setLength(Double length) {
        this.length = length;
    }

    public LinkedList<Node> getRoadPath() {
        return roadPath;
    }

    public void setRoadPath(LinkedList<Node> roadPath) {
        this.roadPath = roadPath;
    }

    /**
     * 通过用dijstra算法求得最短路径
     * @param m 一条路的起点
     * @param n 一条路的终点
     * @return  找到路径，值为false，否则，为true
     */
    public  boolean getShortPath(int m, int n){
        ArrayList<VertexNode> vertexNodes = graph.getVertexNodes();
        int size = vertexNodes.size();
        double []dist = new double[size];
        int []path = new int[size];
        int []S = new int[size];
        double MINdis;
        int i,j,u = 0;
        int v = GraphUtil.getIndex(vertexNodes,m);
        //传入的两个点相等，返回false
        if(m == n){
            return false;
        }
        //初始化
        for(i = 0;i<size;i++){
            dist[i] = 99999;
            path[i] = -1;
            S[i] = 0;
        }
        //如果这个点是不能到达别的地方的话，就直接返回false
        if(v==-1 || vertexNodes.get(v).getArcNodes().isEmpty()){
            return false;
        }else {
            for(i = 0;i<vertexNodes.get(v).getArcNodes().size();i++){
                ArcNode arcNode = vertexNodes.get(v).getArcNodes().get(i);
                int index = GraphUtil.getIndex(vertexNodes,arcNode.getNodeID());
                dist[index] = arcNode.getLength();
                path[index] = v;
            }
        }

        S[v] = 1;path[v] = v;
        for(i = 0;i<size;i++){
            MINdis = 99999;
            for(j = 0;j<size;j++){
                if(S[j] == 0 && dist[j]<MINdis){
                    u = j;
                    MINdis = dist[j];
                }
            }

            S[u] = 1;
            for(j = 0;j<vertexNodes.get(u).getArcNodes().size();j++){
                ArcNode arcNode = vertexNodes.get(u).getArcNodes().get(j);
                int index = GraphUtil.getIndex(vertexNodes,arcNode.getNodeID());
                if(index != -1 && dist[u]+arcNode.getLength()<dist[index]){
                    dist[index] = dist[u]+arcNode.getLength();
                    path[index] = u;
                }
            }
        }
        //paths两个点的路径
        LinkedList<Integer> paths = DisPath(dist,path,S,m,n);
        //路径长度为0返回false
        //否则将路径添加到roadPath
        if(paths.size() == 0){
            return false;
        }else {
            for(i = 0;i<paths.size();i++){
                Node node = new Node();
                int index = GraphUtil.getIndex(vertexNodes,paths.get(i));
                node.setNodeID(vertexNodes.get(index).getNodeID());
                node.setLon(vertexNodes.get(index).getLon());
                node.setLat(vertexNodes.get(index).getLat());
                roadPath.add(node);
            }
        }
        return true;
    }

    /**
     *
     * @param dist  通过dijstra算法求得两个点的距离的数组
     * @param path  通过通过dijstra算法求得两个点的路径
     * @param S 有那些点找到最短路径
     * @param m 起点ID
     * @param n 终点ID
     * @return  起点到终点的路径
     */
    public  LinkedList<Integer> DisPath(double dist[],int path[],int S[],int m,int n){
        ArrayList<VertexNode> vertexNodes = graph.getVertexNodes();
        int i,j,k,d;
        int []apath = new int[vertexNodes.size()];
        LinkedList<Integer> paths =new LinkedList<>();
        int v = GraphUtil.getIndex(vertexNodes,m);
        int s = GraphUtil.getIndex(vertexNodes,n);
        for(i = 0;i<vertexNodes.size();i++){
            if(S[i] == 1 && i!= v && i == s){
                length = dist[i];
                d = 0;
                apath[d] = i;
                k = path[i];
                if(k == -1){
                    System.out.println("无路径");
                    return paths;
                }
                else {
                    while (k!=v){
                        d++;
                        apath[d] = k;
                        k = path[k];
                    }
                    d++;apath[d] = v;
                    paths.add(vertexNodes.get(apath[d]).getNodeID());
                    for(j = d-1;j>=0;j--){
                        paths.add(vertexNodes.get(apath[j]).getNodeID());
                    }
                }

            }
            else if(S[i] == 0 && i == s){
                return paths;
            }

        }
        return paths;
    }

}
