package Utils;

import Entity.*;
import Mapper.RoadMapper;
import sun.print.PathGraphics;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

public class GraphUtil {
    /**
     * 通过图的顶点的集合以及路网关系的集合建立图
     * @param vertexNodes   图的顶点
     * @param roadNodes 路网关系
     * @return  图的对象
     */
    public static Graph  RoadGraph(ArrayList<VertexNode> vertexNodes, LinkedList<RoadNode> roadNodes){
        Graph graph = new Graph(vertexNodes.size());
        for(int i = 0;i<roadNodes.size();i++){
            RoadNode roadNode = roadNodes.get(i);
            //找到该条路的起点在头节点中的位置
            int index = GraphUtil.getIndex(vertexNodes,roadNode.getNodeID());
            //邻接点
            ArcNode arcNode = new ArcNode(roadNode.getLinkNodeID(),roadNode.getLon2(),roadNode.getLat2(),roadNode.getLength());
            if(index != -1){
                VertexNode vertexNode = vertexNodes.get(index);
                LinkedList<ArcNode> arcNodes = vertexNode.getArcNodes();
                arcNodes.add(arcNode);
                vertexNode.setArcNodes(arcNodes);
                vertexNodes.set(index,vertexNode);
            }
        }
        graph.setVertexNodes(vertexNodes);
        return graph;
    }

    /**
     * 跟据车的路径和路网建立的图得到新的图
     * @param nodes 车的路径
     * @param RoadGraph 根据路网关系和全部顶点坐标建立的图
     * @return  新的图
     */
    public static Graph PathGraph(LinkedList<Node> nodes,Graph RoadGraph){
        Graph graph = new Graph(nodes.size());
        ArrayList<VertexNode> vertexNodes = new ArrayList<>();
        //设置顶点坐标
        for(int i = 0;i<nodes.size();i++){
            Node node = nodes.get(i);
            VertexNode vertexNode = new VertexNode();
            vertexNode.setArcNodes(new LinkedList<>());
            vertexNode.setNodeID(node.getNodeID());
            vertexNode.setLon(node.getLon());
            vertexNode.setLat(node.getLat());
            vertexNodes.add(vertexNode);
        }
        for(int i = 0;i<nodes.size()-1;i++){
            Node node1 = nodes.get(i);
            Node node2 = nodes.get(i+1);
            int index = -1;
            //找到该条路的起点在头节点中的位置
            for(int j = 0;j<vertexNodes.size();j++){
                if(vertexNodes.get(j).getNodeID() ==  node1.getNodeID()){
                    index = j;
                    break;
                }
            }
            if(index != -1){
                VertexNode vertexNode = vertexNodes.get(index);
                LinkedList<ArcNode> arcNodes = vertexNode.getArcNodes();
                Double length = GraphUtil.findLeng(RoadGraph,node1.getNodeID(),node2.getNodeID());
                ArcNode arcNode = new ArcNode(node2.getNodeID(),node2.getLon(),node2.getLat(),length);
                arcNodes.add(arcNode);
                vertexNode.setArcNodes(arcNodes);
                vertexNodes.set(index,vertexNode);
            }
        }
        graph.setVertexNodes(vertexNodes);
        return graph;
    }
    /**
     * 找到指定图中起点ID-终点ID的长度
     * @param graph 指定的图
     * @param id1   起点ID
     * @param id2   终点ID
     * @return  路的长度
     */
    public static Double findLeng(Graph graph, int id1, int id2){
        double leng = 0;
        List<VertexNode> vertexNodes = graph.getVertexNodes();
        for(int i = 0;i<vertexNodes.size();i++){
            VertexNode vertexNode = vertexNodes.get(i);
            if(id1 == vertexNode.getNodeID()){
                for(int j = 0;j<vertexNode.getArcNodes().size();j++){
                    if(vertexNode.getArcNodes().get(j).getNodeID() == id2){
                        leng = vertexNode.getArcNodes().get(j).getLength();
                        return leng;
                    }
                }
            }
        }
        return leng;
    }

    /**
     * 找到一个点在顶点集合中的位置
     * @param vertexNodes   顶点坐标
     * @param nodeID    点的ID
     * @return  索引位置
     */
    public static int getIndex(ArrayList<VertexNode> vertexNodes,int nodeID){
        for(int i = 0;i<vertexNodes.size();i++){
            if(nodeID == vertexNodes.get(i).getNodeID()){
                return i;
            }
        }
        return -1;
    }

}
